<?php
/** @var \Icinga\Web\View $this */
/** @var \Icinga\Module\Toplevelview\ViewConfig $view */
$tree = $view->getTree();
if (! $this->compact):
?>
<div class="controls">
    <?= $this->tabs ?>
</div>
<?php endif ?>
<div class="content tlv-content">
    <div class="tlv-header">
        <?= $this->badges($tree->getStatus(), false, true) ?>
        <h1><?= $view->getMeta('name') ?></h1>
        <?= $this->partial('show/actions.phtml', $this) ?>
        <div class="last-refresh"><?= $this->timeAgo($tree->getFetchTime()) ?></div>
    </div>
    <div class="tlv-view-tiles">
        <?php foreach ($tree->getChildren() as $topTile): ?>
            <?= $this->tiles($topTile) ?>
        <?php endforeach; ?>
    </div>
</div>
